//
//  ActivityRecognitionManager.swift
//  activity_rec_flutter
//
//  Created by WOO JIN HWANG on 2021/04/23.
//

import CoreMotion
import Foundation
import SQLite3

typealias ActivityDataHandler = (ActivityData) -> Void

class ActivityRecognitionManager {

  private let motionActivityManager = CMMotionActivityManager()
  private var isRunningService = false

  // Create an instance of UserDefaultsManager
  let userDefaultsManager = UserDefaultsManager.shared

  public func startService(handler: @escaping ActivityDataHandler) {
    if (isRunningService) {
      NSLog("ActivityRecognitionManager The activity recognition service has already started.")
      stopService()
    }

    motionActivityManager.startActivityUpdates(to: .main) { (activity) in
      guard let activity = activity else { return }

      let actData = ActivityData(from: activity)
      //let currentTime = String(Date().timeIntervalSince1970 * 1000)

      //todo write point calculation logic here
      let recordedActivity = self.userDefaultsManager.recordedActivity
      let recordedActivityTime = self.userDefaultsManager.recordedActivityTime

       print("ActivityRecognitionManager activity type: \(actData.type), confidence: \(actData.confidence), startDate: \(actData.startDate)")
       print("ActivityRecognitionManager userDefaultsManager recordedActivity =====> \(recordedActivity)")
       print("ActivityRecognitionManager userDefaultsManager recordedActivityTime =====> \(recordedActivityTime)")

       if(recordedActivity == ""){
        print("ActivityRecognitionManager =====> recordedActivity == null")
        self.userDefaultsManager.recordedActivity = actData.type
        self.userDefaultsManager.recordedActivityTime = actData.startDate

       }else{
       print("ActivityRecognitionManager =====> recordedActivity != null")
        if (actData.type != recordedActivity) {
           print("ActivityRecognitionManager =====> activityData != recordedActivity")

            self.userDefaultsManager.recordedActivity = actData.type
            self.userDefaultsManager.recordedActivityTime = Date()
            let timeIntervalInSeconds = Date().timeIntervalSince(recordedActivityTime)
            let timeIntervalInMinutes = Int(timeIntervalInSeconds / 60)
            print("ActivityRecognitionManager mini difference is \(timeIntervalInSeconds) seconds.")
            print("ActivityRecognitionManager mini difference is \(timeIntervalInMinutes) minutes.")

            let activityType = self.userDefaultsManager.recordedActivity
            print("ActivityRecognitionManager mini activityType \(activityType)")
               switch activityType {
               case "IN_VEHICLE":
                      let points = Int(timeIntervalInMinutes) * 10 // per minute 10 points for vehicle
                      let records =  DatabaseManager.shared.getActivityPoints()
                      let numberOfRecords = records.count
                      print("ActivityRecognitionManager IN_VEHICLE records retrieved: \(numberOfRecords)")
                      // Check if there are any records returned.
                      if let firstRecord = records.first {
                          let vehiclePointsOfFirstRecord = firstRecord.vehiclePoints
                          print("ActivityRecognitionManager IN_VEHICLE points: \(vehiclePointsOfFirstRecord)")
                          // edit record logic
                          DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                       vehiclePoints: firstRecord.vehiclePoints + points,
                                                                       bicyclePoints: firstRecord.bicyclePoints,
                                                                       runningPoints: firstRecord.runningPoints,
                                                                       walkingPoints: firstRecord.walkingPoints,
                                                                       stillPoints: firstRecord.stillPoints)

                      } else {
                          print("ActivityRecognitionManager IN_VEHICLE No records retrieved.")
                          // add record logic
                          DatabaseManager.shared.insertActivityPoints(vehiclePoints:points,
                          bicyclePoints:0,
                          runningPoints:0,
                          walkingPoints:0,
                          stillPoints:0)
                      }

               case "ON_BICYCLE":
                   let points = Int(timeIntervalInMinutes) * 8 // per minute 8 points for bicycle
                   let records =  DatabaseManager.shared.getActivityPoints()
                   let numberOfRecords = records.count
                   print("ActivityRecognitionManager ON_BICYCLE records retrieved: \(numberOfRecords)")
                   // Check if there are any records returned.
                   if let firstRecord = records.first {
                       let vehiclePointsOfFirstRecord = firstRecord.bicyclePoints
                       print("ActivityRecognitionManager ON_BICYCLE points: \(vehiclePointsOfFirstRecord)")
                       // edit record logic
                       DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                   vehiclePoints: firstRecord.vehiclePoints,
                                                                   bicyclePoints: firstRecord.bicyclePoints + points,
                                                                   runningPoints: firstRecord.runningPoints,
                                                                   walkingPoints: firstRecord.walkingPoints,
                                                                   stillPoints: firstRecord.stillPoints)
                   } else {
                       print("ActivityRecognitionManager ON_BICYCLE No records retrieved.")
                       // add record logic
                       DatabaseManager.shared.insertActivityPoints(vehiclePoints:0,
                                                 bicyclePoints:points,
                                                 runningPoints:0,
                                                 walkingPoints:0,
                                                 stillPoints:0)
                   }

               case "RUNNING":
                   let points = Int(timeIntervalInMinutes) * 6 // per minute 6 points for running
                   let records =  DatabaseManager.shared.getActivityPoints()
                   let numberOfRecords = records.count
                   print("ActivityRecognitionManager RUNNING records retrieved: \(numberOfRecords)")
                   // Check if there are any records returned.
                   if let firstRecord = records.first {
                       let vehiclePointsOfFirstRecord = firstRecord.runningPoints
                       print("ActivityRecognitionManager RUNNING points: \(vehiclePointsOfFirstRecord)")
                       // edit record logic
                       DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                  vehiclePoints: firstRecord.vehiclePoints,
                                                                  bicyclePoints: firstRecord.bicyclePoints,
                                                                  runningPoints: firstRecord.runningPoints + points,
                                                                  walkingPoints: firstRecord.walkingPoints,
                                                                  stillPoints: firstRecord.stillPoints)
                   } else {
                       print("ActivityRecognitionManager RUNNING No records retrieved.")
                       // add record logic
                       DatabaseManager.shared.insertActivityPoints(vehiclePoints:0,
                                                                        bicyclePoints:0,
                                                                        runningPoints:points,
                                                                        walkingPoints:0,
                                                                        stillPoints:0)
                   }

               case "WALKING":
                   let points = Int(timeIntervalInMinutes) * 4 // per minute 4 points for walking
                   let records =  DatabaseManager.shared.getActivityPoints()
                   let numberOfRecords = records.count
                   print("ActivityRecognitionManager WALKING records retrieved: \(numberOfRecords)")
                   // Check if there are any records returned.
                   if let firstRecord = records.first {
                       let vehiclePointsOfFirstRecord = firstRecord.walkingPoints
                       print("ActivityRecognitionManager WALKING points: \(vehiclePointsOfFirstRecord)")
                       // edit record logic
                       DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                  vehiclePoints: firstRecord.vehiclePoints,
                                                                  bicyclePoints: firstRecord.bicyclePoints,
                                                                  runningPoints: firstRecord.runningPoints,
                                                                  walkingPoints: firstRecord.walkingPoints + points,
                                                                  stillPoints: firstRecord.stillPoints)
                   } else {
                       print("ActivityRecognitionManager WALKING No records retrieved.")
                       // add record logic
                       DatabaseManager.shared.insertActivityPoints(vehiclePoints:0,
                                                                  bicyclePoints:0,
                                                                  runningPoints:0,
                                                                  walkingPoints:points,
                                                                  stillPoints:0)
                   }

               case "STILL":
                   let points = Int(timeIntervalInMinutes) * 2 // per minute 2 points for still
                   let records =  DatabaseManager.shared.getActivityPoints()
                   let numberOfRecords = records.count
                   print("ActivityRecognitionManager STILL records retrieved: \(numberOfRecords)")
                   if let firstRecord = records.first {
                       let vehiclePointsOfFirstRecord = firstRecord.stillPoints
                       print("ActivityRecognitionManager STILL points: \(vehiclePointsOfFirstRecord)")
                       // edit record logic
                       DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                  vehiclePoints: firstRecord.vehiclePoints,
                                                                  bicyclePoints: firstRecord.bicyclePoints,
                                                                  runningPoints: firstRecord.runningPoints,
                                                                  walkingPoints: firstRecord.walkingPoints,
                                                                  stillPoints: firstRecord.stillPoints + points)
                   } else {
                       print("ActivityRecognitionManager STILL No records retrieved.")
                       // add record logic
                       DatabaseManager.shared.insertActivityPoints(vehiclePoints:0,
                                                                  bicyclePoints:0,
                                                                  runningPoints:0,
                                                                  walkingPoints:0,
                                                                  stillPoints:points)
                   }


               default:
                   print("UNKNOWN activity")
               }


        }
       }

      DatabaseManager.shared.insertActivityData(type: actData.type, confidence: actData.confidence,startDate:actData.startDate)

      handler(ActivityData(from: activity))
    }
    isRunningService = true
  }

  public func stopService() {
    motionActivityManager.stopActivityUpdates()
    isRunningService = false
  }
}
