//
//  MethodCallHandlerImpl.swift
//  activity_rec_flutter
//
//  Created by WOO JIN HWANG on 2021/04/23.
//

import Foundation

class MethodCallHandlerImpl {
  private let methodChannel: FlutterMethodChannel
  private let permissionManager = PermissionManager()
  
  init(messenger: FlutterBinaryMessenger) {
    self.methodChannel = FlutterMethodChannel(
      name: "activity_rec_flutter/method", binaryMessenger: messenger)
    self.methodChannel.setMethodCallHandler(methodCallHandler)
  }
  
  private func methodCallHandler(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    switch call.method {
      case "checkPermission":
        permissionManager.checkMotionActivityPermission { permissionRequestResult in
          result(permissionRequestResult.rawValue)
        }
        break
      case "queryActivityData":
        if let arguments = call.arguments as? [String: Any],
                       let startDateMilliseconds = arguments["startDate"] as? Int64 {
                        let startDate = Date(timeIntervalSince1970: TimeInterval(startDateMilliseconds) / 1000)
                        let endDate = Date() // Current date
                        let activityManager = ActivityManager()

                        activityManager.queryActivityData(from: startDate, to: endDate)
                    }
                    result(true)
        break
      default:
        result(FlutterMethodNotImplemented)
    }
  }
}
