import Foundation

class UserDefaultsManager {
    static let shared = UserDefaultsManager()

    private let autoTrackingKey = "_prefAutoTrackingOnOff"
    private let recordedActivityKey = "prefRecordedActivity"
    private let recordedActivityTimeKey = "prefRecordedActivityTime"

    private let currentInVehicleKey = "_prefCurrentInVehiclePoints"
    private let prevInVehicleKey = "_prefPrevInVehiclePoints"
    private let currentOnCycleKey = "_prefCurrentOnByciclePoints"
    private let prevOnCycleKey = "_prefPrevOnByciclePoints"
    private let currentRunningKey = "_prefCurrentRunningPoints"
    private let prevRunningKey = "_prefPrevRunningPoints"
    private let currentWalkingKey = "_prefCurrentWalkingPoints"
    private let prevWalkingKey = "_prefPrevWalkingPoints"
    private let currentStillKey = "_prefCurrentStillPoints"
    private let prevStillKey = "_prefPrevStillPoints"

    private let userDefaults = UserDefaults.standard

    var autoTrackingOnOff: Bool {
        get { userDefaults.bool(forKey: autoTrackingKey) }
        set { userDefaults.set(newValue, forKey: autoTrackingKey) }
    }

    var recordedActivity: String {
        get { userDefaults.string(forKey: recordedActivityKey) ?? "" }
        set { userDefaults.set(newValue, forKey: recordedActivityKey) }
    }

    var recordedActivityTime: Date {
        get {
            if let data = UserDefaults.standard.object(forKey: recordedActivityTimeKey) as? Data,
               let date = NSKeyedUnarchiver.unarchiveObject(with: data) as? Date {
                return date
            }
            // Default value if not found
            return Date() // You may want to return a meaningful default value here
        }
        set {
            let data = NSKeyedArchiver.archivedData(withRootObject: newValue)
            UserDefaults.standard.set(data, forKey: recordedActivityTimeKey)
        }
    }





    var currentInVehiclePoints: Int {
        get { userDefaults.integer(forKey: currentInVehicleKey) }
        set { userDefaults.set(newValue, forKey: currentInVehicleKey) }
    }

    var prevInVehiclePoints: Int {
        get { userDefaults.integer(forKey: prevInVehicleKey) }
        set { userDefaults.set(newValue, forKey: prevInVehicleKey) }
    }

    var currentOnCyclePoints: Int {
        get { userDefaults.integer(forKey: currentOnCycleKey) }
        set { userDefaults.set(newValue, forKey: currentOnCycleKey) }
    }

    var prevOnCyclePoints: Int {
        get { userDefaults.integer(forKey: prevOnCycleKey) }
        set { userDefaults.set(newValue, forKey: prevOnCycleKey) }
    }

    var currentRunningPoints: Int {
        get { userDefaults.integer(forKey: currentRunningKey) }
        set { userDefaults.set(newValue, forKey: currentRunningKey) }
    }

    var prevRunningPoints: Int {
        get { userDefaults.integer(forKey: prevRunningKey) }
        set { userDefaults.set(newValue, forKey: prevRunningKey) }
    }

    var currentWalkingPoints: Int {
        get { userDefaults.integer(forKey: currentWalkingKey) }
        set { userDefaults.set(newValue, forKey: currentWalkingKey) }
    }

    var prevWalkingPoints: Int {
        get { userDefaults.integer(forKey: prevWalkingKey) }
        set { userDefaults.set(newValue, forKey: prevWalkingKey) }
    }

    var currentStillPoints: Int {
        get { userDefaults.integer(forKey: currentStillKey) }
        set { userDefaults.set(newValue, forKey: currentStillKey) }
    }

    var prevStillPoints: Int {
        get { userDefaults.integer(forKey: prevStillKey) }
        set { userDefaults.set(newValue, forKey: prevStillKey) }
    }
}
