import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:activity_rec_flutter/models/activity.dart';
import 'package:activity_rec_flutter/models/activity_confidence.dart';
import 'package:activity_rec_flutter/models/activity_type.dart';
import 'package:activity_rec_flutter/models/permission_request_result.dart';
import 'package:flutter/services.dart';

export 'package:activity_rec_flutter/models/activity.dart';
export 'package:activity_rec_flutter/models/activity_confidence.dart';
export 'package:activity_rec_flutter/models/activity_type.dart';
export 'package:activity_rec_flutter/models/permission_request_result.dart';

/// A class that provides an activity recognition API.
class FlutterActivityRecognition {
  FlutterActivityRecognition._internal();

  /// Instance of [FlutterActivityRecognition].
  static final instance = FlutterActivityRecognition._internal();

  /// Method channel to communicate with the platform.
  final _methodChannel = MethodChannel('activity_rec_flutter/method');

  /// Event channel to communicate with the platform.
  final _eventChannel = EventChannel('activity_rec_flutter/updates');

  /// Gets the activity stream.
  Stream<Activity> get activityStream {
    return _eventChannel.receiveBroadcastStream().map((event) {
      final data = Map<String, dynamic>.from(jsonDecode(event));
      final type = getActivityTypeFromString(data['type']);
      final confidence = getActivityConfidenceFromString(data['confidence']);
      return Activity(type, confidence);
    });
  }

  /// Get previous activiyt data between last stored activity to current activity.
  Future<bool> queryActivityData(DateTime startDate) async {
    try {
      final bool status =
          await _methodChannel.invokeMethod('queryActivityData', {'startDate': startDate.millisecondsSinceEpoch});
      return status;
    } on PlatformException catch (e) {
      print("Failed to query activity data: '${e.message}'.");
      return false;
    }
  }

  /// Check whether activity recognition permission is granted.
  Future<PermissionRequestResult> checkPermission() async {
    final permissionResult = await _methodChannel.invokeMethod('checkPermission');
    return getPermissionRequestResultFromString(permissionResult);
  }

  /// Request activity recognition permission.
  Future<PermissionRequestResult> requestPermission() async {
    final permissionResult = (Platform.isAndroid)
        ? await _methodChannel.invokeMethod('requestPermission')
        : await _methodChannel.invokeMethod('checkPermission');
    return getPermissionRequestResultFromString(permissionResult);
  }
}
